@@ -15,6 +15,33 @@
  */
 namespace fz {
 
+/**
+ * \ brief Small class to return filesystem errors
+ */
+class FZ_PUBLIC_SYMBOL result final
+{
+public:
+	enum error {
+		ok,
+
+		/// Permission denied
+		noperm,
+
+		/// Requested file does not exist or is not a file
+		nofile,
+
+		/// Requested dir does not exist or is not a dir
+		nodir,
+
+		/// Some other error
+		other
+	};
+
+	explicit operator bool() const { return error_ == 0; }
+
+	error error_{};
+};
+
 /**
  * \brief This class can be used to enumerate the contents of local directories and to query
  * the metadata of files.
@@ -67,7 +94,7 @@ public:
 	/// \brief Begins enumerating a directory.
 	///
 	/// \param dirs_only If true, only directories are enumerated.
-	bool begin_find_files(native_string path, bool dirs_only = false);
+	result begin_find_files(native_string path, bool dirs_only = false);
 
 	/// Gets the next file in the directory. Call until it returns false.
 	bool get_next_file(native_string& name);
@@ -87,10 +114,6 @@ public:
 	static native_string get_link_target(native_string const& path);
 
 private:
-#ifndef FZ_WINDOWS
-	void alloc_path_buffer(char const* filename); // Ensures m_raw_path is large enough to hold path and filename
-#endif
-
 	// State for directory enumeration
 	bool dirs_only_{};
 
@@ -100,9 +123,6 @@ private:
 	bool has_next_{};
 	native_string m_find_path;
 #else
-	char* m_raw_path{};
-	char* m_file_part{}; // Points into m_raw_path past the trailing slash of the path part
-	int m_buffer_length{};
 	DIR* dir_{};
 #endif
 };
