@@ -154,6 +154,10 @@ void bucket_base::remove_bucket()
 					parent->buckets_[idx_] = other;
 				}
 				parent->buckets_.pop_back();
+				std::array<rate::type, 2> unspent = gather_unspent_for_removal();
+				for (size_t i = 0; i < 2; ++i) {
+					parent->data_[i].debt_ -= std::min(parent->data_[i].debt_, unspent[i]);
+				}
 				parent->mtx_.unlock();
 				break;
 			}
@@ -308,7 +312,7 @@ void rate_limiter::pay_debt(direction::type const d)
 		size_t weight = weight_ ? weight_ : 1;
 		rate::type debt_reduction = std::min(data.merged_tokens_, data.debt_ / weight);
 		data.merged_tokens_ -= debt_reduction;
-		data.debt_ -= debt_reduction;
+		data.debt_ -= debt_reduction * weight;
 	}
 	else {
 		data.debt_ = 0;
@@ -365,7 +369,7 @@ rate::type rate_limiter::add_tokens(direction::type const d, rate::type tokens,
 	data.overflow_ = 0;
 	scratch_buffer_.clear();
 	for (size_t i = 0; i < buckets_.size(); ++i) {
-		size_t overflow = buckets_[i]->add_tokens(d, data.merged_tokens_, merged_limit);
+		rate::type overflow = buckets_[i]->add_tokens(d, data.merged_tokens_, merged_limit);
 		if (overflow) {
 			data.overflow_ += overflow;
 		}
@@ -406,21 +410,19 @@ rate::type rate_limiter::distribute_overflow(direction::type const d, rate::type
 	rate::type remaining = overflow_sum;
 
 	while (true) {
-		size_t size{};
+		data.unsaturated_ = 0;
 		for (auto idx : scratch_buffer_) {
-			size += buckets_[idx]->unsaturated(d);
-		}
-		data.unsaturated_ = size;
-
-		if (!remaining || scratch_buffer_.empty()) {
-			break;
+			data.unsaturated_ += buckets_[idx]->unsaturated(d);
 		}
 
-		rate::type const extra_tokens = remaining / size;
-		remaining %= size;
+		rate::type const extra_tokens = data.unsaturated_ ? (remaining / data.unsaturated_) : 0;
+		if (data.unsaturated_) {
+			remaining %= data.unsaturated_;
+		}
 		for (size_t i = 0; i < scratch_buffer_.size(); ) {
-			rate::type sub_overflow = buckets_[scratch_buffer_[i]]->distribute_overflow(d, extra_tokens);
-			if (sub_overflow) {
+			auto & bucket = *buckets_[scratch_buffer_[i]];
+			rate::type sub_overflow = bucket.distribute_overflow(d, extra_tokens);
+			if (sub_overflow || !bucket.unsaturated(d)) {
 				remaining += sub_overflow;
 				scratch_buffer_[i] = scratch_buffer_.back();
 				scratch_buffer_.pop_back();
@@ -430,6 +432,10 @@ rate::type rate_limiter::distribute_overflow(direction::type const d, rate::type
 			}
 		}
 		if (!extra_tokens) {
+			data.unsaturated_ = 0;
+			for (auto idx : scratch_buffer_) {
+				data.unsaturated_ += buckets_[idx]->unsaturated(d);
+			}
 			break;
 		}
 	}
@@ -462,6 +468,25 @@ void rate_limiter::update_stats(bool & active)
 	}
 }
 
+std::array<rate::type, 2> rate_limiter::gather_unspent_for_removal()
+{
+	std::array<rate::type, 2> ret = {0, 0};
+
+	for (auto & bucket : buckets_) {
+		scoped_lock l(bucket->mtx_);
+		auto u = bucket->gather_unspent_for_removal();
+		ret[0] += u[0];
+		ret[1] += u[1];
+	}
+
+	for (size_t i = 0; i < 2; ++i) {
+		rate::type debt_reduction = std::min(ret[i], data_[i].debt_);
+		ret[i] -= debt_reduction;
+		data_[i].debt_ -= debt_reduction;
+	}
+
+	return ret;
+}
 
 bucket::~bucket()
 {
@@ -602,4 +627,17 @@ void bucket::consume(direction::type const d, rate::type amount)
 	}
 }
 
+std::array<rate::type, 2> bucket::gather_unspent_for_removal()
+{
+	std::array<rate::type, 2> ret = {0, 0};
+	for (size_t i = 0; i < 2; ++i) {
+		if (data_[i].available_ != rate::unlimited) {
+			ret[i] = data_[i].available_;
+			data_[i].available_ = 0;
+		}
+	}
+
+	return ret;
+}
+
 }
