@@ -29,6 +29,9 @@
   #endif
   #include <poll.h>
   #undef mutex
+  #if HAVE_EVENTFD
+	#include <sys/eventfd.h>
+  #endif
 #endif
 
 #include <assert.h>
@@ -313,6 +316,19 @@ private:
 	bool initialized_{};
 };
 #endif
+
+
+void close_socket_fd(socket::socket_t& fd)
+{
+	if (fd != -1) {
+#ifdef FZ_WINDOWS
+		closesocket(fd);
+#else
+		close(fd);
+#endif
+		fd = -1;
+	}
+}
 }
 
 class socket_thread final
@@ -327,10 +343,6 @@ public:
 	{
 #ifdef FZ_WINDOWS
 		static winsock_initializer init;
-		sync_event_ = WSA_INVALID_EVENT;
-#else
-		pipe_[0] = -1;
-		pipe_[1] = -1;
 #endif
 		for (int i = 0; i < WAIT_EVENTCOUNT; ++i) {
 			triggered_errors_[i] = 0;
@@ -340,16 +352,68 @@ public:
 	~socket_thread()
 	{
 		thread_.join();
+		destroy_sync();
+	}
+
+	int create_sync()
+	{
+#ifdef FZ_WINDOWS
+		if (sync_event_ == WSA_INVALID_EVENT) {
+			sync_event_ = WSACreateEvent();
+		}
+		if (sync_event_ == WSA_INVALID_EVENT) {
+			return 1;
+		}
+#elif defined HAVE_EVENTFD
+		if (event_fd_ == -1) {
+			event_fd_ = eventfd(0, EFD_CLOEXEC|EFD_NONBLOCK);
+			if (event_fd_ == -1) {
+				return errno;
+			}
+#ifndef HAVE_POLL
+			if (event_fd_ >= FD_SETSIZE) {
+				destroy_sync();
+				return EMFILE;
+			}
+#endif
+		}
+#else
+		if (pipe_[0] == -1) {
+			if (!create_pipe(pipe_)) {
+				return errno;
+			}
+
+#ifndef HAVE_POLL
+			if (pipe_[0] >= FD_SETSIZE) {
+				destroy_sync();
+				return EMFILE;
+			}
+#endif
+		}
+#endif
+		return 0;
+	}
+
+
+	void destroy_sync()
+	{
 #ifdef FZ_WINDOWS
 		if (sync_event_ != WSA_INVALID_EVENT) {
 			WSACloseEvent(sync_event_);
 		}
+#elif defined HAVE_EVENTFD
+		if (event_fd_ != -1) {
+			::close(event_fd_);
+			event_fd_ = -1;
+		}
 #else
 		if (pipe_[0] != -1) {
-			close(pipe_[0]);
+			::close(pipe_[0]);
+			pipe_[0] = -1;
 		}
 		if (pipe_[1] != -1) {
-			close(pipe_[1]);
+			::close(pipe_[1]);
+			pipe_[1] = -1;
 		}
 #endif
 	}
@@ -385,9 +449,7 @@ public:
 		// Connect method of socket ensures port is in range
 		port_ = fz::to_string(port);
 
-		start();
-
-		return 0;
+		return start();
 	}
 
 	int start()
@@ -399,35 +461,20 @@ public:
 			wakeup_thread(l);
 			return 0;
 		}
-#ifdef FZ_WINDOWS
-		if (sync_event_ == WSA_INVALID_EVENT) {
-			sync_event_ = WSACreateEvent();
-		}
-		if (sync_event_ == WSA_INVALID_EVENT) {
-			return 1;
-		}
-#else
-		if (pipe_[0] == -1) {
-			if (!create_pipe(pipe_)) {
-				return errno;
-			}
 
-#ifndef HAVE_POLL
-			if (pipe_[0] >= FD_SETSIZE) {
-				::close(pipe_[0]);
-				::close(pipe_[1]);
-				pipe_[0] = -1;
-				pipe_[1] = -1;
-
-				return EMFILE;
-			}
-#endif
+		int res = create_sync();
+		if (res) {
+			return res;
 		}
-#endif
 
 		thread_ = socket_->thread_pool_.spawn([this]() { entry(); });
 
-		return thread_ ? 0 : 1;
+		if (!thread_) {
+			destroy_sync();
+			return EMFILE;
+		}
+
+		return 0;
 	}
 
 	// Cancels select or idle wait
@@ -451,6 +498,13 @@ public:
 
 #ifdef FZ_WINDOWS
 		WSASetEvent(sync_event_);
+#elif defined(HAVE_EVENTFD)
+		uint64_t tmp = 1;
+
+		int ret;
+		do {
+			ret = write(event_fd_, &tmp, 8);
+		} while (ret == -1 && errno == EINTR);
 #else
 		char tmp = 0;
 
@@ -497,26 +551,14 @@ protected:
 		return fd;
 	}
 
-	static void close_socket_fd(socket::socket_t& fd)
-	{
-		if (fd != -1) {
-	#ifdef FZ_WINDOWS
-			closesocket(fd);
-	#else
-			close(fd);
-	#endif
-			fd = -1;
-		}
-	}
-
 	int try_connect_host(addrinfo & addr, sockaddr_u const& bindAddr, scoped_lock & l)
 	{
 		if (socket_->evt_handler_) {
 			socket_->evt_handler_->send_event<hostaddress_event>(socket_->ev_source_, socket::address_to_string(addr.ai_addr, addr.ai_addrlen));
 		}
 
-		socket::socket_t fd = create_socket_fd(addr);
-		if (fd == -1) {
+		socket_->fd_ = create_socket_fd(addr);
+		if (socket_->fd_ == -1) {
 			if (socket_->evt_handler_) {
 				socket_->evt_handler_->send_event<socket_event>(socket_->ev_source_, addr.ai_next ? socket_event_flag::connection_next : socket_event_flag::connection, last_socket_error());
 			}
@@ -525,14 +567,14 @@ protected:
 		}
 
 		if (bindAddr.sockaddr_.sa_family != AF_UNSPEC && bindAddr.sockaddr_.sa_family == addr.ai_family) {
-			(void)::bind(fd, &bindAddr.sockaddr_, sizeof(bindAddr));
+			(void)::bind(socket_->fd_, &bindAddr.sockaddr_, sizeof(bindAddr));
 		}
 
 		auto* s = static_cast<socket*>(socket_);
-		do_set_flags(fd, s->flags_, s->flags_, s->keepalive_interval_);
-		do_set_buffer_sizes(fd, socket_->buffer_sizes_[0], socket_->buffer_sizes_[1]);
+		do_set_flags(socket_->fd_, s->flags_, s->flags_, s->keepalive_interval_);
+		do_set_buffer_sizes(socket_->fd_, socket_->buffer_sizes_[0], socket_->buffer_sizes_[1]);
 
-		int res = ::connect(fd, addr.ai_addr, addr.ai_addrlen);
+		int res = ::connect(socket_->fd_, addr.ai_addr, addr.ai_addrlen);
 		if (res == -1) {
 #ifdef FZ_WINDOWS
 			// Map to POSIX error codes
@@ -549,9 +591,6 @@ protected:
 		}
 
 		while (res == EINPROGRESS) {
-
-			socket_->fd_ = fd;
-
 			bool wait_successful;
 			do {
 				wait_successful = do_wait(WAIT_CONNECT, l);
@@ -561,9 +600,8 @@ protected:
 			} while (wait_successful);
 
 			if (!wait_successful) {
-				close_socket_fd(fd);
 				if (socket_) {
-					socket_->fd_ = -1;
+					close_socket_fd(socket_->fd_);
 				}
 				return -1;
 			}
@@ -577,11 +615,9 @@ protected:
 				socket_->evt_handler_->send_event<socket_event>(socket_->ev_source_, addr.ai_next ? socket_event_flag::connection_next : socket_event_flag::connection, res);
 			}
 
-			close_socket_fd(fd);
-			socket_->fd_ = -1;
+			close_socket_fd(socket_->fd_);
 		}
 		else {
-			socket_->fd_ = fd;
 			static_cast<socket*>(socket_)->state_ = socket_state::connected;
 
 			if (socket_->evt_handler_) {
@@ -775,7 +811,11 @@ protected:
 			}
 #elif defined(HAVE_POLL)
 			pollfd fds[2]{};
+#ifdef HAVE_EVENTFD
+			fds[0].fd = event_fd_;
+#else
 			fds[0].fd = pipe_[0];
+#endif
 			fds[0].events = POLLIN;
 			fds[1].fd = socket_->fd_;
 			fds[1].events = 0;
@@ -795,7 +835,11 @@ protected:
 
 			if (res > 0 && fds[0].revents) {
 				char buffer[100];
+#ifdef HAVE_EVENTFD
+				int damn_spurious_warning = read(event_fd_, buffer, 8);
+#else
 				int damn_spurious_warning = read(pipe_[0], buffer, 100);
+#endif
 				(void)damn_spurious_warning; // We do not care about return value and this is definitely correct!
 			}
 
@@ -860,7 +904,11 @@ protected:
 			FD_ZERO(&readfds);
 			FD_ZERO(&writefds);
 
+#ifdef HAVE_EVENTFD
+			FD_SET(event_fd_, &readfds);
+#else
 			FD_SET(pipe_[0], &readfds);
+#endif
 
 			if (waiting_ & (WAIT_READ | WAIT_ACCEPT)) {
 				FD_SET(socket_->fd_, &readfds);
@@ -869,7 +917,11 @@ protected:
 				FD_SET(socket_->fd_, &writefds);
 			}
 
+#ifdef HAVE_EVENTFD
+			int maxfd = std::max(event_fd_, socket_->fd_) + 1;
+#else
 			int maxfd = std::max(pipe_[0], socket_->fd_) + 1;
+#endif
 
 			l.unlock();
 
@@ -877,9 +929,15 @@ protected:
 
 			l.lock();
 
+#ifdef HAVE_EVENTFD
+			if (res > 0 && FD_ISSET(event_fd_, &readfds)) {
+				char buffer[8];
+				int damn_spurious_warning = read(event_fd_, buffer, 8);
+#else
 			if (res > 0 && FD_ISSET(pipe_[0], &readfds)) {
 				char buffer[100];
 				int damn_spurious_warning = read(pipe_[0], buffer, 100);
+#endif
 				(void)damn_spurious_warning; // We do not care about return value and this is definitely correct!
 			}
 
@@ -1039,10 +1097,12 @@ protected:
 
 #ifdef FZ_WINDOWS
 	// We wait on this using WSAWaitForMultipleEvents
-	WSAEVENT sync_event_;
+	WSAEVENT sync_event_{WSA_INVALID_EVENT};
+#elif defined(HAVE_EVENTFD)
+	int event_fd_{-1};
 #else
 	// A pipe is used to unblock select
-	int pipe_[2];
+	int pipe_[2]{-1, -1};
 #endif
 
 	mutex mutex_;
@@ -1125,6 +1185,7 @@ void socket_base::do_set_event_handler(event_handler* pEvtHandler)
 int socket_base::close()
 {
 	if (!socket_thread_) {
+		close_socket_fd(fd_);
 		return 0;
 	}
 
@@ -1137,7 +1198,7 @@ int socket_base::close()
 
 	socket_thread_->wakeup_thread(l);
 
-	socket_thread::close_socket_fd(fd);
+	close_socket_fd(fd);
 	if (dynamic_cast<socket*>(this)) {
 		static_cast<socket*>(this)->state_ = socket_state::closed;
 	}
@@ -1299,6 +1360,12 @@ bool socket_base::bind(std::string const& address)
 }
 
 
+socket_descriptor::~socket_descriptor()
+{
+	close_socket_fd(fd_);
+}
+
+
 listen_socket::listen_socket(thread_pool & pool, event_handler* evt_handler)
 	: socket_base(pool, evt_handler, this)
 	, socket_event_source(this)
@@ -1374,7 +1441,7 @@ int listen_socket::listen(address_type family, int port)
 			}
 
 			res = last_socket_error();
-			socket_thread::close_socket_fd(fd_);
+			close_socket_fd(fd_);
 		}
 		freeaddrinfo(addressList);
 		if (fd_ == -1) {
@@ -1382,11 +1449,10 @@ int listen_socket::listen(address_type family, int port)
 		}
 	}
 
-	int res = ::listen(fd_, 1);
+	int res = ::listen(fd_, 64);
 	if (res) {
 		res = last_socket_error();
-		socket_thread::close_socket_fd(fd_);
-		fd_ = -1;
+		close_socket_fd(fd_);
 		return res;
 	}
 
@@ -1394,15 +1460,34 @@ int listen_socket::listen(address_type family, int port)
 
 	socket_thread_->waiting_ = WAIT_ACCEPT;
 
-	socket_thread_->start();
+	if (socket_thread_->start()) {
+		state_ = listen_socket_state::none;
+		close_socket_fd(fd_);
+		return EMFILE;
+	}
 
 	return 0;
 }
 
 std::unique_ptr<socket> listen_socket::accept(int &error)
+{
+	socket_descriptor desc = fast_accept(error);
+	if (!desc) {
+		return std::unique_ptr<socket>();
+	}
+
+	auto ret = socket::from_descriptor(std::move(desc), thread_pool_, error);
+	if (!ret) {
+		error = ENOMEM;
+	}
+	return ret;
+}
+
+socket_descriptor listen_socket::fast_accept(int &error)
 {
 	if (!socket_thread_) {
-		return nullptr;
+		error = ENOTSOCK;
+		return socket_descriptor();
 	}
 
 	socket_t fd;
@@ -1427,40 +1512,15 @@ std::unique_ptr<socket> listen_socket::accept(int &error)
 		if (fd >= FD_SETSIZE) {
 			::close(fd);
 			error = EMFILE;
-			return nullptr;
+			return socket_descriptor();
 		}
 #endif
 	}
 
-	if (fd == -1) {
-		error = last_socket_error();
-		return nullptr;
+	if (fd != -1) {
+		do_set_buffer_sizes(fd, buffer_sizes_[0], buffer_sizes_[1]);
 	}
-
-#if defined(SO_NOSIGPIPE) && !defined(MSG_NOSIGNAL)
-	// We do not want SIGPIPE if writing to socket.
-	const int value = 1;
-	setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
-#endif
-
-	set_nonblocking(fd);
-
-	do_set_buffer_sizes(fd, buffer_sizes_[0], buffer_sizes_[1]);
-
-	socket* pSocket = new socket(thread_pool_, nullptr);
-	if (!pSocket->socket_thread_) {
-		error = ENOMEM;
-		delete pSocket;
-		return nullptr;
-	}
-
-	pSocket->state_ = socket_state::connected;
-	pSocket->fd_ = fd;
-	pSocket->host_ = to_native(pSocket->peer_ip());
-	pSocket->socket_thread_->waiting_ = WAIT_READ | WAIT_WRITE;
-	pSocket->socket_thread_->start();
-
-	return std::unique_ptr<socket>(pSocket);
+	return socket_descriptor(fd);
 }
 
 listen_socket_state listen_socket::get_state() const
@@ -1490,6 +1550,42 @@ socket::~socket()
 	detach_thread(l);
 }
 
+std::unique_ptr<socket> socket::from_descriptor(socket_descriptor && desc, thread_pool & pool, int & error)
+{
+	if (!desc) {
+		error = ENOTSOCK;
+		return nullptr;
+	}
+
+	socket_t fd = desc.detach();
+
+#if defined(SO_NOSIGPIPE) && !defined(MSG_NOSIGNAL)
+	// We do not want SIGPIPE if writing to socket.
+	const int value = 1;
+	setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
+#endif
+
+	set_nonblocking(fd);
+
+	auto pSocket = std::make_unique<socket>(pool, nullptr);
+	if (!pSocket->socket_thread_) {
+		error = ENOMEM;
+		pSocket.reset();
+	}
+	else {
+		pSocket->state_ = socket_state::connected;
+		pSocket->fd_ = fd;
+		pSocket->host_ = to_native(pSocket->peer_ip());
+		pSocket->socket_thread_->waiting_ = WAIT_READ | WAIT_WRITE;
+		if (pSocket->socket_thread_->start()) {
+			error = ENOMEM;
+			pSocket.reset();
+		}
+	}
+
+	return pSocket;
+}
+
 int socket::connect(native_string const& host, unsigned int port, address_type family)
 {
 	if (state_ != socket_state::none) {
@@ -1771,6 +1867,25 @@ void socket::set_keepalive_interval(duration const& d)
 	}
 }
 
+void socket::set_flags(int flags, bool enable)
+{
+	if (!socket_thread_) {
+		return;
+	}
+
+	scoped_lock l(socket_thread_->mutex_);
+
+	if (fd_ != -1) {
+		do_set_flags(fd_, enable ? flags : 0, flags & (flags ^ flags_), keepalive_interval_);
+	}
+	if (enable) {
+		flags_ |= flags;
+	}
+	else {
+		flags_ &= ~flags;
+	}
+}
+
 void socket::set_flags(int flags)
 {
 	if (!socket_thread_) {
@@ -1785,4 +1900,56 @@ void socket::set_flags(int flags)
 	flags_ = flags;
 }
 
+socket_layer::socket_layer(event_handler* handler, socket_interface& next_layer, bool event_passthrough)
+	: socket_interface(next_layer.root())
+	, event_handler_(handler)
+	, next_layer_(next_layer)
+	, event_passthrough_(event_passthrough)
+{
+	if (event_passthrough) {
+		next_layer_.set_event_handler(handler);
+	}
+}
+
+socket_layer::~socket_layer()
+{
+	next_layer_.set_event_handler(nullptr);
+	remove_socket_events(event_handler_, this);
+}
+
+void socket_layer::set_event_handler(event_handler* handler)
+{
+	auto old = event_handler_;
+	event_handler_ = handler;
+	change_socket_event_handler(old, handler, this);
+
+	if (event_passthrough_) {
+		next_layer_.set_event_handler(handler);
+	}
+}
+
+void socket_layer::forward_socket_event(socket_event_source* source, socket_event_flag t, int error)
+{
+	if (event_handler_) {
+		(*event_handler_)(socket_event(source, t, error));
+	}
+}
+
+void socket_layer::forward_hostaddress_event(socket_event_source* source, std::string const& address)
+{
+	if (event_handler_) {
+		(*event_handler_)(hostaddress_event(source, address));
+	}
+}
+
+void socket_layer::set_event_passthrough()
+{
+	event_passthrough_ = true;
+	next_layer_.set_event_handler(event_handler_);
+}
+
+int socket_layer::shutdown_read()
+{
+	return next_layer_.shutdown_read();
+}
 }
