@@ -9,6 +9,7 @@ class util_test final : public CppUnit::TestFixture
 {
 	CPPUNIT_TEST_SUITE(util_test);
 	CPPUNIT_TEST(test_random);
+	CPPUNIT_TEST(test_bitscan);
 	CPPUNIT_TEST_SUITE_END();
 
 public:
@@ -16,6 +17,7 @@ public:
 	void tearDown() {}
 
 	void test_random();
+	void test_bitscan();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(util_test);
@@ -28,3 +30,12 @@ void util_test::test_random()
 	CPPUNIT_ASSERT(!first.empty());
 	CPPUNIT_ASSERT(first != second);
 }
+
+void util_test::test_bitscan()
+{
+	CPPUNIT_ASSERT(fz::bitscan(12) == 2);
+	CPPUNIT_ASSERT(fz::bitscan_reverse(12) == 3);
+
+	CPPUNIT_ASSERT(fz::bitscan(0x3000000000000000ull) == 60);
+	CPPUNIT_ASSERT(fz::bitscan_reverse(0x3000000000000000ull) == 61);
+}
